==> App-TimelogTxt-0.03.tar.gz: App-TimelogTxt-0.03/lib/App/TimelogTxt/Stack.pm <==
package App::TimelogTxt::Stack;

use warnings;
use strict;
use autodie;

our $VERSION = '0.03';

sub new
{
    my ($class, $file) = @_;
    die "No file specified.\n" unless $file;
    return bless { file => $file }, $class;
}

sub clear
{
    my ($self) = @_;
    unlink $self->{file};
    return;
}

sub push
{
    my ($self, $task) = @_;
    open my $fh, '>>', $self->{file};
    print {$fh} $task, "\n";
    return;
}

sub pop
{
    my ($self) = @_;
    return unless -f $self->{file};
    return $self->_pop();
}

sub _pop
{
    my ($self) = @_;
    open my $fh, '+<', $self->{file};
    my ($line, $pos) = _find_last_line( $fh );
    return unless defined $line;
    seek( $fh, $pos, 0 );
    truncate( $fh, $pos );
    return $line;
}

sub drop
{
    my ($self, $arg) = @_;
    return unless -f $self->{file};
    if( !defined $arg )
    {
        $self->_pop();
    }
    elsif( lc $arg eq 'all' )
    {
        $self->clear()
    }
    elsif( $arg =~ /^[0-9]+$/ )
    {
        $self->_pop() foreach 1 .. $arg;
    }
    return;
}

sub _find_last_line
{
    my ($fh) = @_;
    my ( $lastpos, $lastline );
    while( my ( $line, $pos ) = _readline_pos( $fh ) )
    {
        ( $lastpos, $lastline ) = ( $pos, $line );
    }
    return unless defined $lastline;
    chomp $lastline;
    return ($lastline, $lastpos);
}

sub _readline_pos
{
    my $fh   = shift;
    my $pos  = tell $fh;
    my $line = <$fh>;
    return ( $line, $pos ) if defined $line;
    return;
}

sub list
{
    my ($self) = @_;
    return unless -f $self->{file};
    open my $fh, '<', $self->{file};
    print reverse <$fh>;
    return;
}

1;
__END__

=head1 NAME

ModName - [One line description of module's purpose here]


=head1 VERSION

This document describes ModName version 0.03


=head1 SYNOPSIS

    use ModName;

=for author to fill in:
    Brief code example(s) here showing commonest usage(s).
    This section will be as far as many users bother reading
    so make it as educational and exeplary as possible.
  
  
=head1 DESCRIPTION

=for author to fill in:
    Write a full description of the module and its features here.
    Use subsections (=head2, =head3) as appropriate.


=head1 INTERFACE 

=for author to fill in:
    Write a separate section listing the public components of the modules
    interface. These normally consist of either subroutines that may be
    exported, or methods that may be called on objects belonging to the
    classes provided by the module.


=head1 DIAGNOSTICS

=for author to fill in:
    List every single error and warning message that the module can
    generate (even the ones that will "never happen"), with a full
    explanation of each problem, one or more likely causes, and any
    suggested remedies.

=over

=item C<< Error message here, perhaps with %s placeholders >>

[Description of error here]

=item C<< Another error message here >>

[Description of error here]

[Et cetera, et cetera]

=back


=head1 CONFIGURATION AND ENVIRONMENT

=for author to fill in:
    A full explanation of any configuration system(s) used by the
    module, including the names and locations of any configuration
    files, and the meaning of any environment variables or properties
    that can be set. These descriptions must also include details of any
    configuration language used.
  
ModName requires no configuration files or environment variables.


=head1 DEPENDENCIES

=for author to fill in:
    A list of all the other modules that this module relies upon,
    including any restrictions on versions, and an indication whether
    the module is part of the standard Perl distribution, part of the
    module's distribution, or must be installed separately. ]

None.


=head1 INCOMPATIBILITIES

=for author to fill in:
    A list of any modules that this module cannot be used in conjunction
    with. This may be due to name conflicts in the interface, or
    competition for system or program resources, or due to internal
    limitations of Perl (for example, many modules that use source code
    filters are mutually incompatible).

None reported.


=head1 BUGS AND LIMITATIONS

=for author to fill in:
    A list of known problems with the module, together with some
    indication Whether they are likely to be fixed in an upcoming
    release. Also a list of restrictions on the features the module
    does provide: data types that cannot be handled, performance issues
    and the circumstances in which they may arise, practical
    limitations on the size of data sets, special cases that are not
    (yet) handled, etc.

No bugs have been reported.

=head1 AUTHOR

G. Wade Johnson  C<< wade@anomaly.org >>


=head1 LICENCE AND COPYRIGHT

Copyright (c) <YEAR>, G. Wade Johnson C<< wade@anomaly.org >>. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself. See L<perlartistic>.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

==> App-TimelogTxt-0.03_2.tar.gz: App-TimelogTxt-0.03_2/lib/App/TimelogTxt/Stack.pm <==
package App::TimelogTxt::Stack;

use warnings;
use strict;
use autodie;

our $VERSION = '0.03';

sub new
{
    my ($class, $file) = @_;
    die "No file specified.\n" unless $file;
    return bless { file => $file }, $class;
}

sub clear
{
    my ($self) = @_;
    unlink $self->{file};
    return;
}

sub push
{
    my ($self, $task) = @_;
    open my $fh, '>>', $self->{file};
    print {$fh} $task, "\n";
    return;
}

sub pop
{
    my ($self) = @_;
    return unless -f $self->{file};
    return $self->_pop();
}

sub _pop
{
    my ($self) = @_;
    open my $fh, '+<', $self->{file};
    my ($line, $pos) = _find_last_line( $fh );
    return unless defined $line;
    seek( $fh, $pos, 0 );
    truncate( $fh, $pos );
    return $line;
}

sub drop
{
    my ($self, $arg) = @_;
    return unless -f $self->{file};
    if( !defined $arg )
    {
        $self->_pop();
    }
    elsif( lc $arg eq 'all' )
    {
        $self->clear()
    }
    elsif( $arg =~ /^[0-9]+$/ )
    {
        $self->_pop() foreach 1 .. $arg;
    }
    return;
}

sub _find_last_line
{
    my ($fh) = @_;
    my ( $lastpos, $lastline );
    while( my ( $line, $pos ) = _readline_pos( $fh ) )
    {
        ( $lastpos, $lastline ) = ( $pos, $line );
    }
    return unless defined $lastline;
    chomp $lastline;
    return ($lastline, $lastpos);
}

sub _readline_pos
{
    my $fh   = shift;
    my $pos  = tell $fh;
    my $line = <$fh>;
    return ( $line, $pos ) if defined $line;
    return;
}

sub list
{
    my ($self, $ofh) = @_;
    $ofh ||= \*STDOUT;
    return unless -f $self->{file};
    open my $fh, '<', $self->{file};
    print {$ofh} reverse <$fh>;
    return;
}

1;
__END__

=head1 NAME

App::TimelogTxt::Stack - Interface to the stack file for the timelog application.

=head1 VERSION

This document describes App::TimelogTxt::Stack version 0.03_1

=head1 SYNOPSIS

    use ModName;

=for author to fill in:
    Brief code example(s) here showing commonest usage(s).
    This section will be as far as many users bother reading
    so make it as educational and exeplary as possible.


=head1 DESCRIPTION

=for author to fill in:
    Write a full description of the module and its features here.
    Use subsections (=head2, =head3) as appropriate.


=head1 INTERFACE

=head2 new( $filename )

=head2 $s->clear()

=head2 $s->push( $task )

=head2 $task = $s->pop()

=head2 $s->drop( $arg )

=head2 $s->list( $fh )

=head1 CONFIGURATION AND ENVIRONMENT

App::TimelogTxt::Stack requires no configuration files or environment variables.

=head1 DEPENDENCIES

autodie.

=head1 INCOMPATIBILITIES

None reported.

=head1 BUGS AND LIMITATIONS

No bugs have been reported.

=head1 AUTHOR

G. Wade Johnson  C<< gwadej@cpan.org >>

=head1 LICENCE AND COPYRIGHT

Copyright (c) 2013, G. Wade Johnson C<< gwadej@cpan.org >>. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself. See L<perlartistic>.

=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.
